/*********************************************************************
 
	File: Camera.h
	Data: 3/18/2008
	Desc: Declares the Camera class.
 
 **********************************************************************/

#ifndef _CAMERA_H
#define _CAMERA_H

#include "Vector3.h"
#include "Point3.h"
#include "Matrix.h"

class Camera
{
public:

	Camera() { Set(Point3f(4.0f, 4.0f, 3.0f), Point3f(0.0f, 0.0f, 0.0f), Vector3f(0.0f, 1.0f, 0.0f)); }
	Camera(const Point3f &eye, const Point3f &center, const Vector3f &upVec) { Set(eye, center, upVec); }

	void Set(const Point3f &eye, const Point3f &center, const Vector3f &upVec)
	{
		pos = eye;
		lookAt = center;

		forward = Normalize(center - eye);
		right = Normalize(forward.Cross(upVec));
		up = Normalize(right.Cross(forward));
	}

	Matrixf GetViewMatrix() const
	{
		Matrixf m;
		m.MakeView(pos, right, up, forward);
		return m;
	}

	void Move(const Vector3f &amt)
	{
		Vector3f dx = right * amt.x;
		Vector3f dz = Vector3f(forward.x, 0.0f, forward.z) * amt.z;

		Vector3f d = dx + dz;

		lookAt = lookAt + d;
		pos = pos + d;
	}

	void Zoom(float d)
	{
		pos = pos + forward * d;
	}

	void RotateRight(float a)
	{
		Matrixf R;
		R.MakeRotationY(a);

		pos = R.Transform(pos);
		forward = Normalize(lookAt - pos);
		right = Normalize(forward.Cross(Vector3f(0,1,0)));
		up = Normalize(right.Cross(forward));
	}

	void RotateUp(float a)
	{
		Matrixf R;
		R.MakeRotation(right, a);

		pos = R.Transform(pos);
		forward = Normalize(lookAt - pos);
		up = Normalize(right.Cross(forward));
	}

	void Get(Point3f *eye, Point3f *center, Vector3f *rightVec, Vector3f *upVec, Vector3f *forwardVec) const
	{
		if(eye)
			*eye = pos;
		if(center)
			*center = lookAt;
		if(rightVec)
			*rightVec = right;
		if(upVec)
			*upVec = up;
		if(forwardVec)
			*forwardVec = forward;
	}

private:
	Point3f pos;      // eye point
	Point3f lookAt;

	Vector3f right;   // x-axis
	Vector3f up;      // y-axis
	Vector3f forward; // z-axis
};

#endif
