/*********************************************************************
 
	File: Color4.h
	Data: 3/18/2008
	Desc: Declares the Color3 class.
 
 **********************************************************************/

#ifndef _COLOR4_H
#define _COLOR4_H

typedef unsigned char byte;

class Color4f
{
public:
	float r, g, b, a;

	Color4f() : r(0.0f), g(0.0f), b(0.0f), a(1.0f) {}
	Color4f(float _r, float _g, float _b) : r(_r), g(_g), b(_b), a(1.0f) {}
	Color4f(float _r, float _g, float _b, float _a) : r(_r), g(_g), b(_b), a(_a) {}

	void Get(byte &_r, byte &_g, byte &_b) const
	{
		_r = (byte)(r * 255.0f);
		_g = (byte)(g * 255.0f);
		_b = (byte)(b * 255.0f);
	}

	void Get(byte &_r, byte &_g, byte &_b, byte &_a) const
	{
		_r = (byte)(r * 255.0f);
		_g = (byte)(g * 255.0f);
		_b = (byte)(b * 255.0f);
		_a = (byte)(a * 255.0f);
	}

	unsigned int Get() const
	{
		byte _r, _g, _b, _a;
		Get(_r, _g, _b, _a);
		
		return (_r << 24) | (_g << 16) | (_b << 8) | _a;
	}

	Color4f operator+(const Color4f &c) const
	{
		return Color4f(r + c.r, g + c.g, b + c.b, a + c.a);
	}

	Color4f operator-(const Color4f &c) const
	{
		return Color4f(r - c.r, g - c.g, b - c.b, a - c.a);
	}

	Color4f operator*(float c) const
	{
		return Color4f(r*c, g*c, b*c, a*c);
	}

	friend Color4f operator*(float c, const Color4f &col)
	{
		return Color4f(c*col.r, c*col.g, c*col.b, c*col.a);
	}
};

namespace Colors
{
	/* Usefull color constants */
	const Color4f Red(1.0f, 0.0f, 0.0f);
	const Color4f Green(0.0f, 0.0f, 1.0f);
	const Color4f Blue(0.0f, 0.0f, 1.0f);
	const Color4f Black(0.0f, 0.0f, 0.0f);
	const Color4f White(1.0f, 1.0f, 1.0f);
};

#endif
