/*********************************************************************
 
	File: Point3.h
	Data: 3/18/2008
	Desc: Declares the Point3 class.
 
 **********************************************************************/

#ifndef _POINT3_H
#define _POINT3_H

#include "Vector3.h"

template<class T>
class Point3
{
public:
	T x, y, z;
	
	Point3() : x((T)0.0), y((T)0.0), z((T)0.0) {}
	Point3(T _x, T _y, T _z) : x(_x), y(_y), z(_z) {}
	Point3(const Vector3<T> &v) : x(v.x), y(v.y), z(v.z) {}
	
	void Set(T _x, T _y, T _z) { x = _x; y = _y; z = _z; }
	
	Point3 operator+(const Point3 &p) const
	{
		return Point3(x + p.x, y + p.y, z + p.z);
	}
	
	Point3 operator+(const Vector3<T> &v) const
	{
		return Point3(x + v.x, y + v.y, z + v.z);
	}
	
	Point3 &operator+=(const Point3 &p)
	{
		x += p.x;
		y += p.y;
		z += p.z;
		return *this;
	}
	
	Vector3<T> operator-(const Point3 &p) const
	{
		return Vector3<T>(x - p.x, y - p.y, z - p.z);
	}

	Point3 operator-() const
	{
		return Point3(-x, -y, -z);
	}
	
	Point3 &operator-=(const Point3 &p)
	{
		x -= p.x;
		y -= p.y;
		z -= p.z;
		return *this;
	}

	T& operator[](int i)
	{
		return ((T*)this)[i];
	}

	T operator[](int i) const
	{
		return ((T*)this)[i];
	}
	
	Point3 operator*(T c) const
	{
		return Point3(c*x, c*y, c*z);
	}
	
	friend Point3 operator*(float c, const Point3 &p)
	{
		return Point3(c*p.x, c*p.y, c*p.z);
	}
};

typedef Point3<float> Point3f;
typedef Point3<double> Point3d;


// Useful function for manipulating Points and vectors together
template<class T>
T Dot(const Vector3<T> &v, const Point3<T> &p)
{
	return v.x*p.x + v.y*p.y + v.z*p.z;
}

#endif
